"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const guard_1 = require("./guard");
const validate_1 = require("./validate");
const vality_1 = require("./vality");
vality_1.vality.string = (0, guard_1.guard)("string", val => (typeof val === "string" ? val : undefined), {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
    match: (val, o) => o.test(val),
});
vality_1.vality.number = (0, guard_1.guard)("number", val => {
    const nr = typeof val === "number" ? val : config_1.config.strict ? NaN : typeof val === "string" ? Number.parseFloat(val) : NaN;
    if (nr > Number.MIN_SAFE_INTEGER && nr < Number.MAX_SAFE_INTEGER)
        return nr;
    return undefined;
}, {
    min: (val, o) => val >= o,
    max: (val, o) => val <= o,
    integer: (val, o) => !o || val % 1 === 0,
});
const y = ["1", 1, "true"];
const n = ["0", 0, "false"];
vality_1.vality.boolean = (0, guard_1.guard)("boolean", val => {
    if (typeof val === "boolean")
        return val;
    if (config_1.config.strict)
        return undefined;
    if (typeof val !== "string" && typeof val !== "number")
        return undefined;
    return y.indexOf(val) !== -1 ? true : n.indexOf(val) !== -1 ? false : undefined;
});
vality_1.vality.date = (0, guard_1.guard)("date", val => {
    if (val instanceof Date)
        return val;
    if (config_1.config.strict)
        return undefined;
    if (typeof val !== "string" && typeof val !== "number")
        return undefined;
    const date = new Date(val);
    if (Number.isNaN(date.getTime()))
        return undefined;
    return date;
}, {
    min: (val, o) => val >= o,
    max: (val, o) => val <= o,
    past: (val, o) => !o || val < new Date(),
    future: (val, o) => !o || val > new Date(),
});
vality_1.vality.literal = lit => (0, guard_1.guard)("literal", (val, options) => {
    if (options.default === true)
        options.default = val;
    return val === lit ? lit : undefined;
});
vality_1.vality.relation = s => (0, guard_1.guard)("relation", val => {
    const r = (0, validate_1.validate)(vality_1.vality.number({
        integer: true,
        min: 0,
    }), val);
    return r.valid ? r.data : undefined;
});
